@@ -38,7 +38,7 @@ class test_Certificate(SecurityCase):
         x = Certificate(CERT1)
 
         x._cert = Mock(name='cert')
-        time_after = datetime.datetime.now() + datetime.timedelta(days=-1)
+        time_after = datetime.datetime.utcnow() + datetime.timedelta(days=-1)
         x._cert.not_valid_after = time_after
 
         assert x.has_expired() is True
@@ -47,7 +47,7 @@ class test_Certificate(SecurityCase):
         x = Certificate(CERT1)
 
         x._cert = Mock(name='cert')
-        time_after = datetime.datetime.now() + datetime.timedelta(days=1)
+        time_after = datetime.datetime.utcnow() + datetime.timedelta(days=1)
         x._cert.not_valid_after = time_after
 
         assert x.has_expired() is False
