@@ -9,11 +9,13 @@ from case import ContextMock
 
 from celery.utils.collections import LimitedSet
 from celery.worker.consumer.agent import Agent
-from celery.worker.consumer.consumer import CLOSE, TERMINATE, Consumer
+from celery.worker.consumer.consumer import (CANCEL_TASKS_BY_DEFAULT, CLOSE,
+                                             TERMINATE, Consumer)
 from celery.worker.consumer.gossip import Gossip
 from celery.worker.consumer.heart import Heart
 from celery.worker.consumer.mingle import Mingle
 from celery.worker.consumer.tasks import Tasks
+from celery.worker.state import active_requests
 
 
 class test_Consumer:
@@ -272,6 +274,39 @@ class test_Consumer:
         errback(Mock(), 6)
         assert error.call_args[0][3] == 'Trying again in 6.00 seconds... (3/3)'
 
+    def test_cancel_long_running_tasks_on_connection_loss(self):
+        c = self.get_consumer()
+        c.app.conf.worker_cancel_long_running_tasks_on_connection_loss = True
+
+        mock_request_acks_late_not_acknowledged = Mock()
+        mock_request_acks_late_not_acknowledged.task.acks_late = True
+        mock_request_acks_late_not_acknowledged.acknowledged = False
+        mock_request_acks_late_acknowledged = Mock()
+        mock_request_acks_late_acknowledged.task.acks_late = True
+        mock_request_acks_late_acknowledged.acknowledged = True
+        mock_request_acks_early = Mock()
+        mock_request_acks_early.task.acks_late = False
+        mock_request_acks_early.acknowledged = False
+
+        active_requests.add(mock_request_acks_late_not_acknowledged)
+        active_requests.add(mock_request_acks_late_acknowledged)
+        active_requests.add(mock_request_acks_early)
+
+        c.on_connection_error_after_connected(Mock())
+
+        mock_request_acks_late_not_acknowledged.cancel.assert_called_once_with(c.pool)
+        mock_request_acks_late_acknowledged.cancel.assert_not_called()
+        mock_request_acks_early.cancel.assert_not_called()
+
+        active_requests.clear()
+
+    def test_cancel_long_running_tasks_on_connection_loss__warning(self):
+        c = self.get_consumer()
+        c.app.conf.worker_cancel_long_running_tasks_on_connection_loss = False
+
+        with pytest.deprecated_call(match=CANCEL_TASKS_BY_DEFAULT):
+            c.on_connection_error_after_connected(Mock())
+
 
 class test_Heart:
 
